/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "mySolidParticleCloud.H"
#include "fvMesh.H"
#include "volFields.H"
#include "interpolationCellPoint.H"
#include "volPointInterpolation.H"
#include "IOPosition.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
    defineParticleTypeNameAndDebug(mySolidParticle, 0);
    defineTemplateTypeNameAndDebug(Cloud<mySolidParticle>, 0);
};

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::mySolidParticleCloud::mySolidParticleCloud
(
    const fvMesh& mesh,
    const word& cloudName
)
:
    Cloud<mySolidParticle>(mesh, cloudName, false),
    mesh_(mesh),
    particleProperties_
    (
        IOobject
        (
            "particleProperties",
            mesh_.time().constant(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    ),
    rhop_(dimensionedScalar(particleProperties_.lookup("rhop")).value()),
    e_(dimensionedScalar(particleProperties_.lookup("e")).value()),
    mu_(dimensionedScalar(particleProperties_.lookup("mu")).value())
{
    mySolidParticle::readFields(*this);
}
// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::mySolidParticleCloud::move(const dimensionedVector& g)
{
    const volScalarField& rho = mesh_.lookupObject<const volScalarField>("rho");
    const volVectorField& U = mesh_.lookupObject<const volVectorField>("U");
    const volScalarField& nu = mesh_.lookupObject<const volScalarField>("nu");

    pointMesh pMesh(U.mesh());
    volPointInterpolation vpi(U.mesh(), pMesh);

    interpolationCellPoint<scalar> rhoInterp(vpi, rho);
    interpolationCellPoint<vector> UInterp(vpi, U);
    interpolationCellPoint<scalar> nuInterp(vpi, nu);

    mySolidParticle::trackData td(*this, rhoInterp, UInterp, nuInterp, g.value());

    // const vectorField& U_ = mesh_.lookupObject<const vectorField>("lagrangian/defaultCloud/U");

    /*
    // Hr vill jag komma t partiklarna och deras egenskaper.
     forAll(particles p1)
      {
           forAll(particles p2)
           {
            while (p1 != p2)
            {
#           include "collision.H"
            }
           }
      }
 */
    Cloud<mySolidParticle>::move(td);   
}

void Foam::mySolidParticleCloud::writeFields() const
{
    mySolidParticle::writeFields(*this);
}


// ************************************************************************* //
